/*****************************************************************
 * Copyright (c) ParaSoft Corporation 1991.  All rights reserved.*
 * THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF ParaSoft       *
 * The copyright notice above does not evidence any              *
 * actual or intended publication of such source code.           *
 *****************************************************************/

#include "extern.h"

Unit *unit_malloc ()
{
	Unit *unit;

	unit = (Unit*) U_malloc (sizeof (Unit));
	
	return unit;
}

/********************************** C H E C K ***************************************/

Node *check_OpenStmt (root)
Node *root;
{
	Node *list = list_invert (root->OpenStmt.IoSpecList);
	Node *nspec, *temp;
	struct S_IoSpec *spec;
	Unit *unit = unit_malloc ();
	char *name, *cons;
	int first;

	if (flag_cubix)
		cubix_io_called = 1;

	root->OpenStmt.unit = unit;

	first = 1;
	for (LIST_LOOP (nspec, list, temp)) {
		spec = &nspec->IoSpec;

		if (spec->Name == NN) {
			if (first) {
				if (unit->unit != NN) {
					io_err_spec_used ("UNIT");
					continue;
				}

				if (!IS_INTEGER (spec->Expr)) {
					io_err_spec_type ("UNIT" , "INTEGER");
					continue;
				}

				unit->unit = spec->Expr;
			}
			else
				fatal (
				"UNIT specifier with keyword omitted must be the first item in OPEN");

			continue;
		}

		name = U_strupper (spec->Name->com.str);

		if (spec->Times != NN) {
			sprintf (serr, "cannot use '*' with specifier %s", name);
			fatal (serr);
			continue;
		}

		if (io_spec_is ("UNIT", name, "INTEGER", spec, unit->unit)) {
			unit->unit = spec->Expr;
		}
		else if (io_spec_is_name ("IOSTAT", name, "INTEGER", spec, unit->iostat)) {
			unit->iostat = spec->Expr;
		}
		else if (io_spec_is ("ERR", name, "INTEGER", spec, unit->err)) {
			if (spec->Expr->com.type != INTEGER_CONSTANT) {
				fatal ("specifier ERR needs a label"); 
				continue;
			}

			unit->err = spec->Expr;

			label_install_ref (spec->Expr);
		}
		else if (io_spec_is ("FILE", name, "CHARACTER", spec, unit->file)) {
			unit->file = spec->Expr;
		}
		else if (io_spec_is ("STATUS", name, "CHARACTER", spec, unit->status)) {
			unit->status = spec->Expr;
		}
		else if (io_spec_is ("ACCESS", name, "CHARACTER", spec, unit->access)) {
			unit->access = spec->Expr;
		}
		else if (io_spec_is ("FORM", name, "CHARACTER", spec, unit->form)) {
			unit->form = spec->Expr;
		}
		else if (io_spec_is ("RECL", name, "INTEGER", spec, unit->recl)) {
			unit->recl = spec->Expr;
		}
		else if (io_spec_is ("BLANK", name, "CHARACTER", spec, unit->blank)) {
			unit->blank = spec->Expr;
		}

/*!!! finish and add flag_77 for next ones */

		else if (io_spec_is ("POSITION", name, "CHARACTER", spec, unit->position)) {
			unit->position = spec->Expr;
		}
		else if (io_spec_is ("ACTION", name, "CHARACTER", spec, unit->action)) {
			unit->action = spec->Expr;
		}
		else if (io_spec_is ("DELIM", name, "CHARACTER", spec, unit->delim)) {
			unit->delim = spec->Expr;
		}
		else if (io_spec_is ("PAD", name, "CHARACTER", spec, unit->pad)) {
			unit->pad = spec->Expr;
		}
		else {
			sprintf (serr, "invalid specifier in OPEN statement: %s", name);
			fatal (serr);
		}

		if (first) {
			first = 0;
		}
	}

	if (unit->unit == NN)
		fatal ("the UNIT specifier is required in an OPEN statement");
	
	return root;
}

Node *check_CloseStmt (root)
Node *root;
{
	Node *list = list_invert (root->CloseStmt.IoSpecList);
	Node *nspec, *temp;
	struct S_IoSpec *spec;
	Unit *unit = unit_malloc ();
	char *name, *cons;
	int first;

	if (flag_cubix)
		cubix_io_called = 1;

	root->CloseStmt.unit = unit;

	first = 1;
	for (LIST_LOOP (nspec, list, temp)) {
		spec = &nspec->IoSpec;

/*!!! maybe check that expressions are only integer or real or character in some cases */

		if (spec->Name == NN) {
			if (first) {
				if (unit->unit != NN) {
					io_err_spec_used ("UNIT");
					continue;
				}

				if (!IS_INTEGER (spec->Expr)) {
					io_err_spec_type ("UNIT", "INTEGER");
					continue;
				}

				unit->unit = spec->Expr;
			}
			else
				fatal (
				"unit specifier with keyword omitted must be the first item in OPEN");

			continue;
		}

		name = U_strupper (spec->Name->com.str);

		if (spec->Times != NN) {
			sprintf (serr, "cannot use '*' with specifier %s", name);
			fatal (serr);
			continue;
		}

		if (io_spec_is ("UNIT", name, "INTEGER", spec, unit->unit)) {
			unit->unit = spec->Expr;
		}
		else if (io_spec_is ("IOSTAT", name, "INTEGER", spec, unit->iostat)) {
			if (spec->Expr->com.type != NAME) {
				fatal ("specifier IOSTAT needs a variable"); 
				continue;
			}

			unit->iostat = spec->Expr;
		}
		else if (io_spec_is ("ERR", name, "INTEGER", spec, unit->err)) {
			if (spec->Expr->com.type != INTEGER_CONSTANT) {
				fatal ("specifier ERR needs a label"); 
				continue;
			}

			unit->err = spec->Expr;

			label_install_ref (spec->Expr);
		}
		else if (io_spec_is ("STATUS", name, "CHARACTER", spec, unit->status)) {
			unit->status = spec->Expr;
		}
		else {
			sprintf (serr, "invalid specifier in CLOSE statement: %s", name);
			fatal (serr);
		}

		if (first) {
			first = 0;
		}
	}

	if (unit->unit == NN)
		fatal ("the UNIT specifier is required in a CLOSE statement");
	
	return root;
}

/***************************** W R I T E **********************************/

write_OpenStmt (node)
Node *node;
{
	if (flag_cubix)
		cubix_write_OpenStmt (node);
	else
		write_pass_childs (node);
}

write_CloseStmt (node)
Node *node;
{
	if (flag_cubix)
		cubix_write_CloseStmt (node);
	else
		write_pass_childs (node);
}

/***************************** C U B I X **********************************/

cubix_write_OpenStmt (node)
Node *node;
{
	Unit *unit = node->OpenStmt.unit;

	otab ();
	onam (cubix_ioret ());
	ochb ('=');
	ostr ("kropen (");

	if (unit->unit != NN)
		write_pass (unit->unit);
#ifdef CHECK
	else
		SYSERR;
#endif

	ochb (',');

	if (unit->iostat != NN)
		write_pass (unit->iostat);
	else
		onam (cubix_ioret ());

	ochb (',');

	if (unit->err != NN)
		ochb ('1');
	else
		ochb ('0');

	ochb (',');

	if (unit->file != NN)
		write_pass (unit->file);
	else
		cubix_ns ();

	ochb (',');

	if (unit->status != NN)
		write_pass (unit->status);
	else
		cubix_ns ();

	ochb (',');

	if (unit->access != NN)
		write_pass (unit->access);
	else
		cubix_ns ();

	ochb (',');

	if (unit->form != NN)
		write_pass (unit->form);
	else
		cubix_ns ();

	ochb (',');

	if (unit->recl != NN)
		write_pass (unit->recl);
	else
		ochb ('0');

	ochb (',');

	if (unit->blank != NN)
		write_pass (unit->blank);
	else
		cubix_ns ();

	och (')');

	if (unit->err != NN) {
		odon ();
		otab ();
		oIF ();
		ochb ('(');
		onam (cubix_ioret ());
		ostr (".ne. 0) goto");
		write_pass (unit->err);
	}
}

cubix_write_CloseStmt (node)
Node *node;
{
	Unit *unit = node->CloseStmt.unit;

	otab ();
	onam (cubix_ioret ());
	ochb ('=');
	ostr ("krclos (");

	if (unit->unit != NN)
		write_pass (unit->unit);
#ifdef CHECK
	else
		SYSERR;
#endif

	ochb (',');

	if (unit->iostat != NN)
		write_pass (unit->iostat);
	else
		onam (cubix_ioret ());

	ochb (',');

	if (unit->err != NN)
		ochb ('1');
	else
		ochb ('0');

	ochb (',');

	if (unit->status != NN)
		write_pass (unit->status);
	else
		cubix_ns ();

	och (')');

	if (unit->err != NN) {
		odon ();
		otab ();
		oIF ();
		ochb ('(');
		onam (cubix_ioret ());
		ostr (".ne. 0) goto");
		write_pass (unit->err);
	}
}
